@@ -34,6 +34,33 @@
 #include <stdlib.h>
 
 #include <sstream>
+#include <algorithm>
+
+#if defined(_WIN32) && !defined(strcasecmp)
+#  define strcasecmp(a,b) stricmp(a,b)
+#endif
+
+void arestest_strtolower(char *dest, const char *src, size_t dest_size)
+{
+  size_t len;
+
+  if (dest == NULL)
+    return;
+
+  memset(dest, 0, dest_size);
+
+  if (src == NULL)
+    return;
+
+  len = strlen(src);
+  if (len >= dest_size)
+    return;
+
+  for (size_t i = 0; i<len; i++) {
+    dest[i] = (char)tolower(src[i]);
+  }
+}
+
 
 namespace ares {
 
@@ -272,8 +299,14 @@ std::string QuestionToString(const std::vector<byte>& packet,
   }
   *len -= (int)enclen;
   *data += enclen;
-  ss << "'" << name << "' ";
+
+  // DNS 0x20 may mix case, output as all lower for checks as the mixed case
+  // is really more of an internal thing
+  char lowername[256];
+  arestest_strtolower(lowername, name, sizeof(lowername));
   ares_free_string(name);
+
+  ss << "'" << lowername << "' ";
   if (*len < NS_QFIXEDSZ) {
     ss << "(too short, len left " << *len << ")";
     return ss.str();
@@ -341,21 +374,21 @@ std::string RRToString(const std::vector<byte>& packet,
     case T_TXT: {
       const byte* p = *data;
       while (p < (*data + rdatalen)) {
-        int len = *p++;
-        if ((p + len) <= (*data + rdatalen)) {
-          std::string txt(p, p + len);
-          ss << " " << len << ":'" << txt << "'";
+        int tlen = *p++;
+        if ((p + tlen) <= (*data + rdatalen)) {
+          std::string txt(p, p + tlen);
+          ss << " " << tlen << ":'" << txt << "'";
         } else {
           ss << "(string too long)";
         }
-        p += len;
+        p += tlen;
       }
       break;
     }
     case T_CNAME:
     case T_NS:
     case T_PTR: {
-      int rc = ares_expand_name(*data, packet.data(), (int)packet.size(), &name, &enclen);
+      rc = ares_expand_name(*data, packet.data(), (int)packet.size(), &name, &enclen);
       if (rc != ARES_SUCCESS) {
         ss << "(error from ares_expand_name)";
         break;
@@ -366,7 +399,7 @@ std::string RRToString(const std::vector<byte>& packet,
     }
     case T_MX:
       if (rdatalen > 2) {
-        int rc = ares_expand_name(*data + 2, packet.data(), (int)packet.size(), &name, &enclen);
+        rc = ares_expand_name(*data + 2, packet.data(), (int)packet.size(), &name, &enclen);
         if (rc != ARES_SUCCESS) {
           ss << "(error from ares_expand_name)";
           break;
@@ -384,7 +417,7 @@ std::string RRToString(const std::vector<byte>& packet,
         unsigned long weight = DNS__16BIT(p + 2);
         unsigned long port = DNS__16BIT(p + 4);
         p += 6;
-        int rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
+        rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
         if (rc != ARES_SUCCESS) {
           ss << "(error from ares_expand_name)";
           break;
@@ -411,7 +444,7 @@ std::string RRToString(const std::vector<byte>& packet,
     }
     case T_SOA: {
       const byte* p = *data;
-      int rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
+      rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
       if (rc != ARES_SUCCESS) {
         ss << "(error from ares_expand_name)";
         break;
@@ -447,22 +480,22 @@ std::string RRToString(const std::vector<byte>& packet,
         p += 4;
         ss << order << " " << pref;
 
-        int len = *p++;
-        std::string flags(p, p + len);
+        int nlen = *p++;
+        std::string flags(p, p + nlen);
         ss << " " << flags;
-        p += len;
+        p += nlen;
 
-        len = *p++;
-        std::string service(p, p + len);
+        nlen = *p++;
+        std::string service(p, p + nlen);
         ss << " '" << service << "'";
-        p += len;
+        p += nlen;
 
-        len = *p++;
-        std::string regexp(p, p + len);
+        nlen = *p++;
+        std::string regexp(p, p + nlen);
         ss << " '" << regexp << "'";
-        p += len;
+        p += nlen;
 
-        int rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
+        rc = ares_expand_name(p, packet.data(), (int)packet.size(), &name, &enclen);
         if (rc != ARES_SUCCESS) {
           ss << "(error from ares_expand_name)";
           break;
@@ -498,7 +531,7 @@ void PushInt16(std::vector<byte>* data, int value) {
   data->push_back((byte)value & 0x00ff);
 }
 
-std::vector<byte> EncodeString(const std::string& name) {
+std::vector<byte> EncodeString(const std::string &name) {
   std::vector<byte> data;
   std::stringstream ss(name);
   std::string label;
@@ -515,23 +548,28 @@ std::vector<byte> EncodeString(const std::string& name) {
   return data;
 }
 
-std::vector<byte> DNSQuestion::data() const {
+std::vector<byte> DNSQuestion::data(const char *request_name, const ares_dns_record_t *dnsrec) const {
   std::vector<byte> data;
-  std::vector<byte> encname = EncodeString(name_);
+  std::vector<byte> encname;
+  if (request_name != nullptr && strcasecmp(request_name, name_.c_str()) == 0) {
+    encname = EncodeString(request_name);
+  } else {
+    encname = EncodeString(name_);
+  }
   data.insert(data.end(), encname.begin(), encname.end());
   PushInt16(&data, rrtype_);
   PushInt16(&data, qclass_);
   return data;
 }
 
-std::vector<byte> DNSRR::data() const {
-  std::vector<byte> data = DNSQuestion::data();
+std::vector<byte> DNSRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSQuestion::data(dnsrec);
   PushInt32(&data, ttl_);
   return data;
 }
 
-std::vector<byte> DNSSingleNameRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSSingleNameRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   std::vector<byte> encname = EncodeString(other_);
   int len = (int)encname.size();
   PushInt16(&data, len);
@@ -539,8 +577,8 @@ std::vector<byte> DNSSingleNameRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSTxtRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSTxtRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   int len = 0;
   for (const std::string& txt : txt_) {
     len += (1 + (int)txt.size());
@@ -553,8 +591,8 @@ std::vector<byte> DNSTxtRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSMxRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSMxRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   std::vector<byte> encname = EncodeString(other_);
   int len = 2 + (int)encname.size();
   PushInt16(&data, len);
@@ -563,8 +601,8 @@ std::vector<byte> DNSMxRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSSrvRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSSrvRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   std::vector<byte> encname = EncodeString(target_);
   int len = 6 + (int)encname.size();
   PushInt16(&data, len);
@@ -575,8 +613,8 @@ std::vector<byte> DNSSrvRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSUriRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSUriRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   int len = 4 + (int)target_.size();
   PushInt16(&data, len);
   PushInt16(&data, prio_);
@@ -585,16 +623,16 @@ std::vector<byte> DNSUriRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSAddressRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSAddressRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   int len = (int)addr_.size();
   PushInt16(&data, len);
   data.insert(data.end(), addr_.begin(), addr_.end());
   return data;
 }
 
-std::vector<byte> DNSSoaRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSSoaRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   std::vector<byte> encname1 = EncodeString(nsname_);
   std::vector<byte> encname2 = EncodeString(rname_);
   int len = (int)encname1.size() + (int)encname2.size() + 5*4;
@@ -609,23 +647,81 @@ std::vector<byte> DNSSoaRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSOptRR::data() const {
-  std::vector<byte> data = DNSRR::data();
-  int len = 0;
+const ares_dns_rr_t *fetch_rr_opt(const ares_dns_record_t *rec)
+{
+  size_t i;
+  for (i = 0; i < ares_dns_record_rr_cnt(rec, ARES_SECTION_ADDITIONAL); i++) {
+    const ares_dns_rr_t *rr =
+      ares_dns_record_rr_get_const(rec, ARES_SECTION_ADDITIONAL, i);
+
+    if (ares_dns_rr_get_type(rr) == ARES_REC_TYPE_OPT) {
+      return rr;
+    }
+  }
+  return NULL;
+}
+
+std::vector<byte> DNSOptRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte>    data = DNSRR::data(dnsrec);
+  int len                   = 0;
+  std::vector<byte>    cookie;
+  const ares_dns_rr_t *rr  = fetch_rr_opt(dnsrec);
+  size_t               passed_cookie_len = 0;
+  const unsigned char *passed_cookie = NULL;
+
+  ares_dns_rr_get_opt_byid(rr, ARES_RR_OPT_OPTIONS, ARES_OPT_PARAM_COOKIE,
+                           &passed_cookie, &passed_cookie_len);
+
+  /* Error out if we expected a server cookie but didn't get one, or if the
+   * passed in server cookie doesn't match our expected value */
+  if (expect_server_cookie_ &&
+      (passed_cookie_len <= 8 ||
+       passed_cookie_len - 8 != server_cookie_.size() ||
+       memcmp(passed_cookie + 8, server_cookie_.data(), server_cookie_.size()) != 0
+      )
+     ) {
+    data.clear();
+    return data;
+  }
+
+  /* See if we should be applying a server cookie */
+  if (server_cookie_.size() && passed_cookie_len >= 8) {
+    /* If client cookie was provided to test framework, we are overwriting
+     * the one received from the client.  This is likely to test failure
+     * scenarios */
+    if (client_cookie_.size()) {
+      cookie.insert(cookie.end(), client_cookie_.begin(), client_cookie_.end());
+    } else {
+      cookie.insert(cookie.end(), passed_cookie, passed_cookie+8);
+    }
+    cookie.insert(cookie.end(), server_cookie_.begin(), server_cookie_.end());
+  }
+
+  if (cookie.size()) {
+    len += 4 + (int)cookie.size();
+  }
   for (const DNSOption& opt : opts_) {
     len += (4 + (int)opt.data_.size());
   }
+
   PushInt16(&data, len);
   for (const DNSOption& opt : opts_) {
     PushInt16(&data, opt.code_);
     PushInt16(&data, (int)opt.data_.size());
     data.insert(data.end(), opt.data_.begin(), opt.data_.end());
   }
+
+  if (cookie.size()) {
+    PushInt16(&data, ARES_OPT_PARAM_COOKIE);
+    PushInt16(&data, (int)cookie.size());
+    data.insert(data.end(), cookie.begin(), cookie.end());
+  }
+
   return data;
 }
 
-std::vector<byte> DNSNaptrRR::data() const {
-  std::vector<byte> data = DNSRR::data();
+std::vector<byte> DNSNaptrRR::data(const ares_dns_record_t *dnsrec) const {
+  std::vector<byte> data = DNSRR::data(dnsrec);
   std::vector<byte> encname = EncodeString(replacement_);
   int len = (4 + 1 + (int)flags_.size() + 1 + (int)service_.size() + 1 + (int)regexp_.size() + (int)encname.size());
   PushInt16(&data, len);
@@ -641,7 +737,7 @@ std::vector<byte> DNSNaptrRR::data() const {
   return data;
 }
 
-std::vector<byte> DNSPacket::data() const {
+std::vector<byte> DNSPacket::data(const char *request_name, const ares_dns_record_t *dnsrec) const {
   std::vector<byte> data;
   PushInt16(&data, qid_);
   byte b = 0x00;
@@ -669,19 +765,35 @@ std::vector<byte> DNSPacket::data() const {
   PushInt16(&data, count);
 
   for (const std::unique_ptr<DNSQuestion>& question : questions_) {
-    std::vector<byte> qdata = question->data();
+    std::vector<byte> qdata = question->data(request_name, dnsrec);
+    if (qdata.size() == 0) {
+      data.clear();
+      return data;
+    }
     data.insert(data.end(), qdata.begin(), qdata.end());
   }
   for (const std::unique_ptr<DNSRR>& rr : answers_) {
-    std::vector<byte> rrdata = rr->data();
+    std::vector<byte> rrdata = rr->data(dnsrec);
+    if (rrdata.size() == 0) {
+      data.clear();
+      return data;
+    }
     data.insert(data.end(), rrdata.begin(), rrdata.end());
   }
   for (const std::unique_ptr<DNSRR>& rr : auths_) {
-    std::vector<byte> rrdata = rr->data();
+    std::vector<byte> rrdata = rr->data(dnsrec);
+    if (rrdata.size() == 0) {
+      data.clear();
+      return data;
+    }
     data.insert(data.end(), rrdata.begin(), rrdata.end());
   }
   for (const std::unique_ptr<DNSRR>& rr : adds_) {
-    std::vector<byte> rrdata = rr->data();
+    std::vector<byte> rrdata = rr->data(dnsrec);
+    if (rrdata.size() == 0) {
+      data.clear();
+      return data;
+    }
     data.insert(data.end(), rrdata.begin(), rrdata.end());
   }
   return data;
